package cbase

import (
	"go_dreamfactory/lego/core"
)

//基础模块对象
type ModuleBase struct {
	comps []core.IModuleComp
}

//此方法可上层重构 返回自定义的参数载体接口对象 用于底层自动序列化参数使用
func (this *ModuleBase) NewOptions() (options core.IModuleOptions) {
	return new(ModuleOptions)
}

//模块初始化接口 上层实现可以更具需求重构扩展接口 通过此接口模块可以拿到 当前运行的服务对象以及实现的模块对象和模块参数对象
//此函数底层实现 初始化模块的组件系统
func (this *ModuleBase) Init(service core.IService, module core.IModule, options core.IModuleOptions) (err error) {
	module.OnInstallComp()
	for _, v := range this.comps {
		err = v.Init(service, module, v, options)
		if err != nil {
			return
		}
	}
	return
}

//暗转组件接口 上层模块实现 徐涛重构此方法 注册模块组件
func (this *ModuleBase) OnInstallComp() {

}

//模块启动接口 启动时可以处理一些外部调用相关的事情
func (this *ModuleBase) Start() (err error) {
	for _, v := range this.comps {
		err = v.Start()
		if err != nil {
			return
		}
	}
	return
}

//模块独立协程 模块可以更具需要自行重构
func (this *ModuleBase) Run(closeSig chan bool) (err error) {

	return
}

//模块消息接口
func (this *ModuleBase) Destroy() (err error) {
	for _, v := range this.comps {
		err = v.Destroy()
		if err != nil {
			return
		}
	}
	return
}

//注册模块组件
func (this *ModuleBase) RegisterComp(comp core.IModuleComp) interface{} {
	this.comps = append(this.comps, comp)
	return comp
}
