package modules

import (
	"go_dreamfactory/lego/base"
	"go_dreamfactory/lego/core"
	"go_dreamfactory/lego/core/cbase"
	"go_dreamfactory/lego/sys/log"
	"go_dreamfactory/sys/configure"
)

///配置管理基础组件
type MCompConfigure struct {
	cbase.ModuleCompBase
	S base.IRPCXService //rpc服务对象
	M IModule           //当前业务模块
}

//组件初始化接口
func (this *MCompConfigure) Init(service core.IService, module core.IModule, comp core.IModuleComp, options core.IModuleOptions) (err error) {
	this.ModuleCompBase.Init(service, module, comp, options)
	this.S = service.(base.IRPCXService)
	this.M = module.(IModule)
	return
}

//加载一个配置文件
func (this *MCompConfigure) LoadConfigure(name string, fn interface{}) (err error) {
	return configure.RegisterConfigure(name, fn)
}

//加载多个配置文件
func (this *MCompConfigure) LoadMultiConfigure(confs map[string]interface{}) (err error) {
	for k, v := range confs {
		err = configure.RegisterConfigure(k, v)
		if err != nil {
			log.Errorf("配置文件:%s解析失败!", k)
			break
		}
	}
	return
}

//读取配置数据
func (this *MCompConfigure) GetConfigure(name string) (v interface{}, err error) {
	return configure.GetConfigure(name)
}
