package equipment

import (
	"fmt"
	"go_dreamfactory/modules"
	cfg "go_dreamfactory/sys/configure/structs"

	"go_dreamfactory/lego/core"
	"go_dreamfactory/lego/sys/log"
)

const (
	game_equip        = "game_equip.json"            //装备信息表
	equip_attrlibrary = "game_equipattrlibrary.json" //装备属性配置表
	equip_intensify   = "game_equipintensify.json"   //装备等级消耗表
	equip_suit        = "game_equipsuit.json"        //装备套装表
)

///背包配置管理组件
type configureComp struct {
	modules.MCompConfigure
}

//组件初始化接口
func (this *configureComp) Init(service core.IService, module core.IModule, comp core.IModuleComp, options core.IModuleOptions) (err error) {
	this.ModuleCompBase.Init(service, module, comp, options)
	this.LoadConfigure(game_equip, cfg.NewGame_equip)
	this.LoadConfigure(equip_attrlibrary, cfg.NewGame_equipAttrlibrary)
	return
}

//获取装备配置数据
func (this *configureComp) GetEquipmentConfigure() (configure *cfg.Game_equip, err error) {
	var (
		v  interface{}
		ok bool
	)
	if v, err = this.GetConfigure(game_equip); err != nil {
		log.Errorf("err:%v", err)
		return
	} else {
		if configure, ok = v.(*cfg.Game_equip); !ok {
			err = fmt.Errorf("%T no is *cfg.Game_equipment", v)
			log.Errorf("err:%v", err)
			return
		}
	}
	return
}

//获取装备配置数据
func (this *configureComp) GetEquipmentConfigureById(equipmentId int32) (configure *cfg.Game_equipData, err error) {
	var (
		v  interface{}
		ok bool
	)
	if v, err = this.GetConfigure(game_equip); err != nil {
		log.Errorf("err:%v", err)
		return
	} else {
		if configure, ok = v.(*cfg.Game_equip).GetDataMap()[equipmentId]; !ok {
			err = fmt.Errorf("EquipmentConfigure not found:%d ", equipmentId)
			log.Errorf("err:%v", err)
			return
		}
	}
	return
}

//获取装备属性表
func (this *configureComp) GetEquipmentAttrlibraryConfigure() (configure *cfg.Game_equipAttrlibrary, err error) {
	var (
		v  interface{}
		ok bool
	)
	if v, err = this.GetConfigure(equip_attrlibrary); err != nil {
		log.Errorf("err:%v", err)
		return
	} else {
		if configure, ok = v.(*cfg.Game_equipAttrlibrary); !ok {
			err = fmt.Errorf("%T no is *cfg.Game_equipment", v)
			log.Errorf("err:%v", err)
			return
		}
	}
	return
}

//获取属性词列表
func (this *configureComp) GetEquipmentAttrlibraryConfigureByKey(key int32) (configure *cfg.Game_equipAttrlibraryData, err error) {
	var (
		v  interface{}
		ok bool
	)
	if v, err = this.GetConfigure(equip_attrlibrary); err != nil {
		log.Errorf("err:%v", err)
		return
	} else {
		if configure, ok = v.(*cfg.Game_equipAttrlibrary).GetDataMap()[key]; !ok {
			err = fmt.Errorf("EquipmentConfigure GetEquipmentAttrlibraryConfigureByKey not found:%d ", key)
			log.Errorf("err:%v", err)
			return
		}
	}
	return
}

//获取属性词列表
func (this *configureComp) GetEquipmentAttrlibraryConfigureById(Id int32) (configure []*cfg.Game_equipAttrlibraryData, err error) {
	var (
		v interface{}
	)
	if v, err = this.GetConfigure(equip_attrlibrary); err != nil {
		log.Errorf("err:%v", err)
		return
	} else {
		configure = make([]*cfg.Game_equipAttrlibraryData, 0)
		for _, v1 := range v.(*cfg.Game_equipAttrlibrary).GetDataList() {
			if v1.Libraryid == Id {
				configure = append(configure, v1)
			}
		}
	}
	return
}

//获取武器等级消耗表
func (this *configureComp) GetEquipmentIntensifyConfigure() (configure *cfg.Game_equipIntensify, err error) {
	var (
		v  interface{}
		ok bool
	)
	if v, err = this.GetConfigure(equip_intensify); err != nil {
		log.Errorf("err:%v", err)
		return
	} else {
		if configure, ok = v.(*cfg.Game_equipIntensify); !ok {
			err = fmt.Errorf("%T no is *cfg.Game_equipment", v)
			log.Errorf("err:%v", err)
			return
		}
	}
	return
}

//获取武器等级消耗表
func (this *configureComp) GetEquipmentIntensifyConfigureById(Id int32) (configure *cfg.Game_equipIntensifyData, err error) {
	var (
		v  interface{}
		ok bool
	)
	if v, err = this.GetConfigure(equip_intensify); err != nil {
		log.Errorf("err:%v", err)
		return
	} else {
		if configure, ok = v.(*cfg.Game_equipIntensify).GetDataMap()[Id]; !ok {
			err = fmt.Errorf("EquipmentConfigure not found:%d ", Id)
			log.Errorf("err:%v", err)
			return
		}
	}
	return
}
