package hero

import (
	"go_dreamfactory/comm"
	"go_dreamfactory/lego/sys/log"
	"go_dreamfactory/pb"
)

//参数校验
func (this *apiComp) StrengthenUplvCheck(session comm.IUserSession, req *pb.HeroStrengthenUplvReq) (result map[string]interface{}, code comm.ErrorCode) {
	if req.HeroObjID == "" {
		code.Code = pb.ErrorCode_ReqParameterError
		return
	}
	var (
		curLv    int32
		curExp   int32 // 当前英雄的经验
		costGold int32 // 当前需要消耗金币的数量
		addExp   int32 // 需要增加的经验
		curGold  int32
		//atn      = map[string]interface{}{}
	)

	_hero, err := this.moduleHero.modelHero.moduleHero.GetHero(session.GetUserId(), req.HeroObjID) // 校验升级的对象是否存在

	if err != 0 {
		code.Code = pb.ErrorCode_HeroNoExist
		return
	}
	_expHero, err := this.moduleHero.modelHero.moduleHero.GetHero(session.GetUserId(), req.ExpCardID) // 校验需要消耗经验卡牌的对象是否存在

	if err != 0 {
		code.Code = pb.ErrorCode_HeroNoExist
		return
	}
	if req.Amount <= 0 { // 消耗数量校验
		code.Code = pb.ErrorCode_HeroNoExist
		return
	}
	curLv = _hero.Lv
	curExp = _hero.Exp // 当前英雄的经验

	// 查询 本次消耗会获得多少经验
	expConf := this.moduleHero.configure.GetHeroExp(_expHero.HeroID)
	if expConf != nil {
		addExp = expConf.Heroexp * req.Amount
	}
	if _expHero.SameCount <= req.Amount { // 消耗经验卡片数量不足
		code.Code = pb.ErrorCode_HeroNoEnough
		return
	}
	// 校验当前能不能升级

	if _hero.Lv >= _hero.Star*comm.HeroStarLvRatio { // 达到最大等级
		code.Code = pb.ErrorCode_HeroMaxLv
		return
	}

	var maxLv int32 // 校验等级达到上限
	maxLv = _hero.Star * comm.HeroStarLvRatio
	_data := this.moduleHero.configure.GetHeroLv(curLv)
	if _data != nil {
		if maxLv >= _hero.Lv && curExp >= _data.Heroexp[0].N { // 加经验之前校验是否达到最大等级
			code.Code = pb.ErrorCode_HeroMaxLv
			return
		}
		curExp += addExp // 先把经验加上
		for {            // 死循环判断一键升级

			if maxLv >= _hero.Lv && curExp >= _data.Heroexp[0].N { // 设置最大经验和等级
				curLv = maxLv
				curExp = _data.Heroexp[0].N
				break
			}
			if _data.Heroexp[0].N > curExp { // 经验不够升级则不能执行升级操作
				break
			} else { // 升级操作
				curExp -= _data.Heroexp[0].N
				curLv += 1 // 经验够了 那么等级+1
				_data = this.moduleHero.configure.GetHeroLv(curLv)
				if _data == nil { // 等级加失败了 回到原来的等级
					curLv -= 1
					break
				}
				costGold += _data.Gold[0].N // 统计 升级需要消耗金币的数量
			}
		}
		// 金币消耗判断
		curGold = this.user.QueryAttributeValue(session.GetUserId(), "gold")
		if curGold < costGold {
			code.Code = pb.ErrorCode_GoldNoEnough
		}
	} else {
		code.Code = pb.ErrorCode_HeroNoExist
		return
	}

	// 校验金币消耗
	result = map[string]interface{}{
		"costGold": costGold,
		"curExp":   curExp,
		"curLv":    curLv,
		"addExp":   addExp,
		//"atn":      atn,
	}
	return
}

/// 英雄升级
func (this *apiComp) StrengthenUplv(session comm.IUserSession, agrs map[string]interface{}, req *pb.HeroStrengthenUplvReq) (code pb.ErrorCode) {
	var (
		curLv    int32
		curExp   int32 // 当前英雄的经验
		costGold int32 // 当前需要消耗金币的数量
		addExp   int32 // 需要增加的经验
		//atn      = map[string]interface{}{}
	)
	defer func() {
		if code == pb.ErrorCode_Success {
			session.SendMsg(string(this.moduleHero.GetType()), StrengthenUplv, &pb.HeroStrengthenUplvResp{})
		}
	}()
	costGold = agrs["costGold"].(int32)
	curLv = agrs["curLv"].(int32)
	curExp = agrs["curExp"].(int32)
	addExp = agrs["addExp"].(int32)
	log.Debugf("升级后当前等级: %d,经验: %d,需要消耗的金币: %d,增加的经验: %d", curLv, curExp, costGold, addExp)
	// 执行升级逻辑
	code = this.moduleHero.modelHero.moduleHero.AddCardExp(session.GetUserId(), req.HeroObjID, addExp) // 加经验
	if code != pb.ErrorCode_Success {
		return
	}
	// 消耗道具
	code = this.user.AddAttributeValue(session.GetUserId(), "gold", -costGold) // 减少金币
	if code != pb.ErrorCode_Success {
		return
	}
	// 删除经验卡
	code = this.moduleHero.modelHero.moduleHero.DelCard(req.ExpCardID, req.Amount)
	if code != pb.ErrorCode_Success {
		return
	}

	return
}
