package user

import (
	"go_dreamfactory/comm"
	"go_dreamfactory/pb"
	"go_dreamfactory/utils"
)

func (this *apiComp) CreateCheck(session comm.IUserSession, req *pb.UserCreateReq) (result map[string]interface{}, code comm.ErrorCode) {
	result = make(map[string]interface{})
	self := &pb.DBUser{}
	err := this.module.modelUser.Get(session.GetUserId(), self)
	if err != nil {
		code = comm.ErrorCode{Code: pb.ErrorCode_DBError}
		return
	}
	result["self"] = self
	return
}

//创角
func (this *apiComp) Create(session comm.IUserSession, result map[string]interface{}, req *pb.UserCreateReq) (code pb.ErrorCode) {
	defer utils.TraceFunc(session.GetUserId(), string(this.module.GetType()), UserSubTypeCreate, req, nil)

	defer func() {
		err := session.SendMsg(string(this.module.GetType()), UserSubTypeCreate, &pb.UserCreateRsp{})
		if err != nil {
			code = pb.ErrorCode_SystemError
		}
	}()

	if self, ok := result["self"]; ok {
		v := self.(*pb.DBUser)
		if v.Name != "" {
			code = pb.ErrorCode_RoleCreated
			return
		}
	}

	update := map[string]interface{}{
		"name": req.NickName,
	}

	err := this.module.modelUser.Change(session.GetUserId(), update)
	if err != nil {
		code = pb.ErrorCode_DBError
		return
	}

	//初始化英雄卡
	defaultHero := []int32{15001, 25001} //TODO 从配置中读取
	err = this.hero.CreateHero(session.GetUserId(), defaultHero...)
	if err != nil {
		code = pb.ErrorCode_HeroInitCreat
		return
	}

	return
}
