package main

import (
	"flag"
	"fmt"
	"go_dreamfactory/modules/friend"
	"go_dreamfactory/modules/hero"
	"go_dreamfactory/modules/items"
	"go_dreamfactory/modules/mail"
	"go_dreamfactory/modules/user"
	"go_dreamfactory/services"
	"go_dreamfactory/sys/cache"
	"go_dreamfactory/sys/db"

	"go_dreamfactory/lego"
	"go_dreamfactory/lego/base/rpcx"
	"go_dreamfactory/lego/core"
	"go_dreamfactory/lego/sys/log"
)

/*
	服务类型:worker
	服务描述:处理梦工厂的具体业务需求，包含 user,pack,mail,friend...功能业务模块
*/
var (
	conf = flag.String("conf", "./conf/worker.yaml", "获取需要启动的服务配置文件") //启动服务的Id
)

/*服务启动的入口函数*/
func main() {
	flag.Parse()
	s := NewService(
		rpcx.SetConfPath(*conf),
		rpcx.SetVersion("1.0.0.0"),
	)
	s.OnInstallComp( //装备组件
		services.NewGateRouteComp(), //此服务需要接受用户的消息 需要装备网关组件
	)
	lego.Run(s, //运行模块
		user.NewModule(),
		items.NewModule(),
		mail.NewModule(),
		friend.NewModule(),
		hero.NewModule(),
	)

}

func NewService(ops ...rpcx.Option) core.IService {
	s := new(Service)
	s.Configure(ops...)
	return s
}

//worker 的服务对象定义
type Service struct {
	services.ServiceBase
}

//初始化worker需要的一些系统工具
func (this *Service) InitSys() {
	this.ServiceBase.InitSys()
	//缓存系统
	if err := cache.OnInit(this.GetSettings().Sys["cache"]); err != nil {
		panic(fmt.Sprintf("init sys.cache err: %s", err.Error()))
	} else {
		log.Infof("init sys.cache success!")
	}
	//存储系统
	if err := db.OnInit(this.GetSettings().Sys["db"]); err != nil {
		panic(fmt.Sprintf("init sys.db err: %s", err.Error()))
	} else {
		log.Infof("init sys.db success!")
	}
}
